@@ -57,30 +57,31 @@ class MicroReader(MicroConventions):
         return self.request_get_json(method='live', arg='summary::' + name)
         # res = requests.get(self.base_url + '/live/summary::' + name)
 
-    def get_lagged(self,name):
-        return self.request_get_json(method='lagged',arg=name)
+    def get_lagged(self,name, count=1000):
+        return self.request_get_json(method='lagged',arg=name, data={'count':count-1})
 
-    def get_lagged_values_and_times(self, name):
-        lagged = self.get_lagged(name=name)
+    def get_lagged_values_and_times(self, name, count=1000):
+        """ Preferred method """
+        lagged = self.get_lagged(name=name, count=count)
         lagged_values = [l[1] for l in lagged]
         lagged_times  = [l[0] for l in lagged]
         return lagged_values, lagged_times
 
-    def get_lagged_values(self, name):
+    def get_lagged_values(self, name:str, count:int=1000):
         """ Retrieve lagged values of a time series
         :param name:    cop.json   z1~cop.json   z2~cop~qp.json
         :return: [ float ]
         """
-        return self.request_get_json(method='live', arg='lagged_values::' + name)
-        # res = requests.get(self.base_url + '/live/lagged_values::' + name)
+        lagged_values, lagged_times = self.get_lagged_values_and_times(name=name,count=count)
+        return lagged_values
 
-    def get_lagged_times(self, name):
+    def get_lagged_times(self, name:str, count:int=1000) -> list:
         """ Retrieve lagged times
         :param name:    cop.json   z1~cop.json   z2~cop~qp.json
         :return: [ float ]
         """
-        return self.request_get_json(method='live', arg='lagged_times::' + name)
-        # res = requests.get(self.base_url + '/live/lagged_times::' + name)
+        lagged_values, lagged_times = self.get_lagged_values_and_times(name=name, count=count)
+        return lagged_values
 
     def get_delayed_value(self, name: str, delay: int):
         """ Retrieve quarantined value.
